SystemOrganization addCategory: #'NXTalk-Kernel'!
SystemOrganization addCategory: #'NXTalk-Building'!
SystemOrganization addCategory: #'NXTalk-Collections'!
SystemOrganization addCategory: #'NXTalk-Tests'!

!SmallInteger methodsFor: '*nxtalk' stamp: 'mab 4/29/2008 16:18'!
asNXTSmallInteger
	^ ((self bitAnd: 16r8000) >> 1 bitOr: (self bitAnd: 16r3FFF)) bitOr: 16r8000! !

RBProgramNodeVisitor subclass: #NXTAbstractCodeCompiler
	instanceVariableNames: 'literals bytecodes half arguments temporaries compilerContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTAbstractCodeCompiler class methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:54'!
bytecodes
	^ Dictionary new
		at: #nop put: 0;
		at: #send put: 1;
		at: #selfSend put: 2;
		at: #push put: 3;
		at: #popEmpty put: 4;
		at: #return put: 5;
		at: #pop put: 6;
		at: #dup put: 7;
		yourself! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 20:06'!
acceptAssignmentNode: anAssignmentNode
	| pushNeeded |
	self visitNode: anAssignmentNode value.
	pushNeeded := anAssignmentNode parent isSequence not or: [ self lastStatement ].
	pushNeeded ifTrue: [ self addBytecode: #dup ].
	self addBytecode: #pop.
	self addVariable: anAssignmentNode variable name.
	^ pushNeeded! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/15/2008 14:35'!
acceptBlockNode: aBlockNode
	| block |
	block := NXTBlockCompiler new
		compilerContext: self;
		compile: aBlockNode.
	self addBytecode: #push.
	self image addObject: block.
	self addLiteral: block.
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 15:35'!
acceptCascadeNode: cascadeNode
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 15:45'!
acceptLiteralNode: aLiteralNode
	| literal |
	self addBytecode: #push.
	aLiteralNode value ifNil: [
		literal := self image specialObjectAt: #nil ].
	(aLiteralNode value isKindOf: Boolean) ifTrue: [
		aLiteralNode value 
			ifTrue: [ literal := self image specialObjectAt: #true ]
			ifFalse: [ literal := self image specialObjectAt: #false ]].
	aLiteralNode value isString ifTrue: [
		literal := NXTImageSymbol new symbol: aLiteralNode value.
		self image addObject: literal.
	].
	aLiteralNode value isInteger ifTrue: [
		literal := aLiteralNode value ].
	self addLiteral: literal.
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 15:35'!
acceptMessageNode: aMessageNode
	self primitive ifNotNil: [ ^ self ].
	aMessageNode arguments do: [:each | self visitNode: each].
	(aMessageNode receiver isVariable and: [ aMessageNode receiver name = 'self']) 
		ifTrue: [ self addBytecode: #selfSend ]
		ifFalse: [
			(aMessageNode isCascaded not or: [aMessageNode isFirstCascaded]) 
			ifTrue: [self visitNode: aMessageNode receiver].
			self addBytecode: #send.
		].
	self addLiteral: (self image symbolAt: aMessageNode selector).
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:47'!
acceptReturnNode: aReturnNode
	(aReturnNode value isVariable and: [ aReturnNode value name = 'self' ]) ifFalse: [
		super acceptReturnNode: aReturnNode.
		self addBytecode: #return ].
	^ false! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 15:34'!
acceptVariableNode: variableNode
	self addBytecode: #push.
	self addVariable: variableNode name.
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:38'!
addArgument: index
	self addBytecodeArgument: (index bitAnd: 16r1F)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:39'!
addBytecode: aSymbol
	| bytecode |
	bytecode := self class bytecodes at: aSymbol.
	"Bytecodes only have 4 bits, thus they get packed"
	half 
		ifFalse: [ self bytecodes add: bytecode << 4. half := true ]
		ifTrue: [ self bytecodes at: self bytecodes size put: (self bytecodes last bitOr: bytecode). half := false ].! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:39'!
addBytecodeArgument: aNumber
	"Arguments always have 8 bits, thus they could be split over byte boundaries"
	half 
		ifFalse: [ self bytecodes add: aNumber ]
		ifTrue: [ self bytecodes at: self bytecodes size put: (self bytecodes last bitOr: aNumber >> 4).
			self bytecodes add: (aNumber bitAnd: 16rF) << 4 ].! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:39'!
addInstVar: index
	self addBytecodeArgument: ((index bitAnd: 16r1F) bitOr: 1 << 5)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 19:16'!
addLiteral: literal
	self addBytecodeArgument: (16r80 bitOr: (self mapLiteral: literal))! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:40'!
addTemporary: index
	self addBytecodeArgument: ((index bitAnd: 16r1F) bitOr: 2 << 5)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 18:36'!
addVariable: aName
	self ifArgument: aName doWithIndex: [ :index |
		^ self addArgument: index ].
	self ifTemporary: aName doWithIndex: [ :index |
		^ self addTemporary: index ].
	self ifInstVar: aName doWithIndex: [ :index |
		^ self addInstVar: index ].
	self error: 'Unknown variable'.! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 19:26'!
allArguments
	^ arguments! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 19:12'!
allTemporaries
	^ temporaries! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:12'!
arguments
	^ arguments! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:12'!
arguments: anObject
	arguments := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
bytecodes
	^ bytecodes! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
bytecodes: anObject
	bytecodes := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:17'!
compilerContext
	^ compilerContext! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:17'!
compilerContext: anObject
	compilerContext := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:15'!
half
	^ half! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:15'!
half: anObject
	half := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:33'!
ifArgument: aName doWithIndex: aBlock
	| index |
	index := self allArguments indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 19:27'!
ifInstVar: aName doWithIndex: aBlock
	| index |
	index := self instVarNames indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:13'!
ifTemporary: aName doWithIndex: aBlock
	| index |
	index := self allTemporaries indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:17'!
image
	^ self compilerContext image! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 19:28'!
initialize
	literals := OrderedCollection new.
	bytecodes := OrderedCollection new.
	arguments := OrderedCollection new.
	temporaries := OrderedCollection new.
	half := false.! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:18'!
instVarNames
	^ self compilerContext instVarNames! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 20:05'!
lastStatement
	^ false! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
literals
	^ literals! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
literals: anObject
	literals := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:44'!
mapLiteral: aLiteral
	^ (self literals indexOf: aLiteral ifAbsent: [ self literals add: aLiteral. self literals size ]) - 1! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 15:37'!
primitive
	^ nil! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
temporaries
	^ temporaries! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
temporaries: anObject
	temporaries := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:44'!
visitArgument: argumentNode
	self arguments add: argumentNode name! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:47'!
visitTemporaryVariable: aNode
	self temporaries add: aNode name.! !

NXTAbstractCodeCompiler subclass: #NXTBlockCompiler
	instanceVariableNames: 'lastStatement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 20:04'!
acceptSequenceNode: aSequenceNode 
	self visitTemporaryVariables: aSequenceNode temporaries.
	aSequenceNode statements allButLastDo: [:each | (self visitNode: each) ifTrue: [self addBytecode: #popEmpty]].
	self lastStatement: true.
	self visitNode: (aSequenceNode statements last).
	half ifTrue: [ self addBytecode: #nop ].! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:27'!
allArguments
	^ self arguments, self compilerContext allArguments! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:27'!
allTemporaries
	^ self temporaries, self compilerContext allTemporaries! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 15:33'!
compile: aBlockNode
	self visitNode: aBlockNode body.
	^ (NXTImageCompiledBlock 
		bytecodes: self bytecodes
		literals: self literals)
		tempCount: self temporaries size! !

!NXTBlockCompiler methodsFor: 'accessing' stamp: 'mab 5/7/2008 17:44'!
initialize
	super initialize.
	lastStatement := false.! !

!NXTBlockCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 20:04'!
lastStatement
	^ lastStatement! !

!NXTBlockCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 20:04'!
lastStatement: anObject
	lastStatement := anObject! !

NXTAbstractCodeCompiler subclass: #NXTMethodCompiler
	instanceVariableNames: 'primitive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 14:34'!
acceptPragmaNode: aPragmaNode
	aPragmaNode pragma keyword = #NXTPrimitive: ifTrue: [
		self primitive: (aPragmaNode pragma argumentAt: 1) ].! !

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 17:07'!
acceptSequenceNode: aSequenceNode 
	self primitive ifNotNil: [ ^ nil ].
	self visitTemporaryVariables: aSequenceNode temporaries.
	aSequenceNode statements do: [:each | (self visitNode: each) ifTrue: [self addBytecode: #popEmpty]].
	half ifTrue: [ self addBytecode: #nop ].! !

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 18:29'!
compileMethod: aCompiledMethod
	| ast |
	ast := SqueakParser parseMethod: aCompiledMethod getSource.
	self visitNode: ast.
	self primitive ifNil: [ ^ (NXTImageCompiledMethod 
		bytecodes: self bytecodes 
		literals: self literals)
		tempCount: self temporaries size ].
	^ NXTImageCompiledMethod primitive: self primitive! !

!NXTMethodCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:37'!
primitive
	^ primitive! !

!NXTMethodCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:08'!
primitive: anObject
	primitive := anObject! !

Object subclass: #NXTClassCompiler
	instanceVariableNames: 'image compiledClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 16:09'!
compile: aClass
	| classObject method methodDictionary |
	self compiledClass: aClass.
	classObject := NXTImageObject new.
	self image addObject: classObject.
	methodDictionary := NXTImageObject new.
	self image addObject: methodDictionary.
	classObject instanceVariables add: methodDictionary.
	aClass selectors do: [ :selector |
		method := NXTMethodCompiler new
			compilerContext: self;
			compileMethod: (aClass compiledMethodAt: selector).
		self image addObject: method.
		methodDictionary instanceVariables add: (self image symbolAt: selector).
		methodDictionary instanceVariables add: method.
	].
	^ classObject! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
compiledClass
	^ compiledClass! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
compiledClass: anObject
	compiledClass := anObject! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
image
	^ image! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
image: anObject
	image := anObject! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:22'!
instVarNames
	^ self compiledClass allInstVarNames! !

Object subclass: #NXTImage
	instanceVariableNames: 'objects specialObjects symbols classes paddings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImage class methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 15:01'!
specialClasses
	^ {
	#NXTSmallInteger
	}! !

!NXTImage class methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 00:01'!
specialObjects
	^ {
	#System -> #NXTSystem.
	#nil -> #NXTUndefinedObject.
	#true -> #NXTTrue. 
	#false -> #NXTFalse.
	}! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/25/2008 23:50'!
addObject: anNXTObject
	^  self objects add: anNXTObject! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 17:20'!
classAt: aClassName
	^ classes at: aClassName ifAbsentPut: [ NXTClassCompiler new
		image: self;
		compile: (Smalltalk at: aClassName) ]! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/29/2008 15:28'!
includeClasses
	| changed |
	changed := true.
	[ changed ] whileTrue: [
		changed := false.
		self objects do: [ :object |
			(object isInteger not and: [ object nxtClass isSymbol ]) ifTrue: [
				changed := true.
				object nxtClass: (self classAt: object nxtClass) ]]].! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 15:02'!
includeSpecialClasses
	self class specialClasses do: [ :class |
		self classAt: class ].! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 00:32'!
includeSpecialObjects
	self class specialObjects do: [ :assoc |
		self specialObjectAt: assoc key ].! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 5/6/2008 13:06'!
initialize
	objects := OrderedCollection new.
	specialObjects := Dictionary new.
	symbols := Dictionary new.
	classes := Dictionary new.
	paddings := 0.! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 4/28/2008 23:31'!
isSpecialObject: aSymbol
	^ self class specialObjects detect: [ :each |
		each key = aSymbol ] ifNone: [ false ].! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
objects
	^ objects! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
objects: anObject
	objects := anObject! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 5/16/2008 20:35'!
specialObjectAt: aSymbol
	| specialObject specialClass|
	^ specialObjects at: aSymbol ifAbsentPut: [
		specialObject := NXTImageObject new.
		specialClass := self specialObjectClassAt: aSymbol.
		(Smalltalk at: specialClass) instVarNames size timesRepeat: [
			specialObject instanceVariables add: (self specialObjectAt: #nil)].
		specialObject nxtClass: specialClass.
		self addObject: specialObject ]! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 4/28/2008 23:36'!
specialObjectClassAt: aSymbol
	^ (self class specialObjects detect: [ :each | each key = aSymbol ]) value! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
specialObjects
	^ specialObjects! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
specialObjects: anObject
	specialObjects := anObject! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:24'!
symbolAt: aSymbol
	^ symbols at: aSymbol ifAbsentPut: [ self addObject: (NXTImageSymbol new symbol: aSymbol) ]! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/12/2008 14:25'!
writeHeaderTo: aByteStream
	aByteStream littleEndianWord: 16r4242.
	aByteStream littleEndianWord: self objects size.! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 5/6/2008 13:07'!
writeObjectsTo: aByteStream withMapper: mapper
	self objects do: [ :each | 
		each writeOn: aByteStream withMapper: mapper.
		aByteStream position odd ifTrue: [ "padding"
			paddings := paddings + 1.
			aByteStream nextPut: 0 ]].! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/29/2008 15:05'!
writeSpecialObjectsTo: aByteStream withMapper: mapper
	aByteStream 
		littleEndianWord: (mapper idFor: (self specialObjects at: #runSymbol)).
	self class specialObjects do: [ :objectToClass |
		aByteStream littleEndianWord: (mapper idFor: (self specialObjectAt: objectToClass key)) ].
	self class specialClasses do: [ :class |
		aByteStream littleEndianWord: (mapper idFor: (self classAt: class)) ].! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/29/2008 15:04'!
writeToStream: aByteStream
	| mapper |
	self includeSpecialObjects.
	self includeSpecialClasses.
	self includeClasses.
	mapper := NXTObjectMapper on: self objects.
	self writeHeaderTo: aByteStream.
	self writeObjectsTo: aByteStream withMapper: mapper.
	self writeSpecialObjectsTo: aByteStream withMapper: mapper.! !

Object subclass: #NXTImageBuilder
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageBuilder class methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 02:52'!
standardCategories
	^ #('NXTalk-Kernel' 'NXTalk-Collections')! !

!NXTImageBuilder class methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:06'!
systemClasses
	^ #(objectClass nilClass classClass arrayClass symbolClass methodClass integerClass bigintegerClass frameClass stringClass doubleClass)! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 03:00'!
compileClasses
	self class standardCategories do: [ :category |
		(Smalltalk organization listAtCategoryNamed: category) do: [ :class |
			self compileClass: class ]].! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:53'!
createMetaclasses
	| metaclass |
	metaclass := self newClass.
	metaclass nxtClass: self newClass.
	"Metaclass class class = Metaclass"
	metaclass nxtClass nxtClass: metaclass.
	^ metaclass! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:24'!
createSpecialObjects
	self image nil: self newObject.
	self image metaclass: self createMetaclasses.
	self createSystemClasses.
	self image nil nxtClass: (self image specialObjects at: #nilClass).
	self initializeSystemClasses.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 03:06'!
createStandardImage
	self image: NXTImage new.
	self createSpecialObjects.
	self compileClasses.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:06'!
createSystemClasses
	self class systemClasses do: [ :symbol |
		self image specialObjects at: symbol put: self newSystemClass ].
	! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 5/17/2008 09:41'!
helloWorldImageOn: aByteStream
	self image: NXTImage new.
	self image specialObjects at: #runSymbol put: (self image symbolAt: #run).
	self image writeToStream: aByteStream.

! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 02:50'!
image
	^ image! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 02:50'!
image: anObject
	image := anObject! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:39'!
initializeSystemClass: classSymbol superclass: superclassSymbol name: nameSymbol
	| class |
	class := self image specialObjects at: classSymbol.
	class nxtSuperclass: (self image specialObjects at: superclassSymbol).
	(self image specialObjects at: #Smalltalk) at: nameSymbol put: class.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 01:00'!
initializeSystemClasses
	self initializeSystemClass: #objectClass superclass: #nil name: #Object.
	self initializeSystemClass: #classClass superclass: #objectClass name: #Class.
	self initializeSystemClass: #metaclass superclass: #classClass name: #Metaclass.
	self initializeSystemClass: #nilClass superclass: #objectClass name: #UndefinedObject.
	self initializeSystemClass: #arrayClass superclass: #objectClass name: #Array.
	self initializeSystemClass: #methodClass superclass: #arrayClass name: #Method.
	self initializeSystemClass: #symbolClass superclass: #objectClass name: #Symbol.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:52'!
newClass
	^  self image addObject: NXTImageClass new! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:52'!
newObject
	^  self image addObject: NXTImageObject new! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:56'!
newSystemClass
	| systemClass |
	systemClass := self newClass.
	systemClass nxtClass: self newClass.
	systemClass nxtClass nxtClass: self image metaclass.
	^ systemClass! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:36'!
objectForClass: anNXTClass
	| class |
	class := NXTImageClass new.
	class nxtClass: self newMetaclass.
	
	^ class! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:17'!
specialObjects
	^ self image specialObjects! !

Object subclass: #NXTImageObject
	instanceVariableNames: 'nxtClass instanceVariables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

NXTImageObject subclass: #NXTImageClass
	instanceVariableNames: 'methodDictionary instanceVariableNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageClass methodsFor: 'accessing' stamp: 'mab 4/17/2008 13:51'!
methodDictionary
	^ methodDictionary! !

!NXTImageClass methodsFor: 'accessing' stamp: 'mab 4/17/2008 13:51'!
methodDictionary: anObject
	methodDictionary := anObject! !

NXTImageObject subclass: #NXTImageCompiledMethod
	instanceVariableNames: 'literals bytecodes primitive tempCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

NXTImageCompiledMethod subclass: #NXTImageCompiledBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageCompiledBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 17:34'!
initialize
	super initialize.
	nxtClass := #NXTBlock! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/22/2008 14:14'!
bytecodes: bytecodes literals: literals
	^self new 
		bytecodes: bytecodes;
		literals: literals! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 18:50'!
fromSelector: aSelector class: aClass within: anNXTImage 
	| aCompiledMethod |
	aCompiledMethod := aClass compiledMethodAt: aSelector.
	^ NXTMethodCompiler new
		image: anNXTImage ;
		instVarNames: aClass allInstVarNames ;
		compileMethod: aCompiledMethod! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 14:41'!
primitive: aNumber
	^ self new primitive: aNumber! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
bytecodes
	^ bytecodes! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
bytecodes: anObject
	bytecodes := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:20'!
computeSize
	^ self primitive
		ifNil: [ 2 + self bytecodes size + (2 * self literals size) ]
		ifNotNil: [ 2 ]! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 17:24'!
elementSize
	^ 1! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
literals
	^ literals! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
literals: anObject
	literals := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:03'!
methodHeader
	^ self primitive 
		ifNil: [ (self literals size bitAnd: 16r7F) << 8 bitOr: self tempCount << 3 ]
		ifNotNil: [ 1 << 15 bitOr: (self primitive bitAnd: 16r7FFF) ]! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:40'!
primitive
	^ primitive! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:40'!
primitive: anObject
	primitive := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:01'!
tempCount
	^ tempCount! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:01'!
tempCount: anObject
	tempCount := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 5/6/2008 15:01'!
writeDataOn: aByteStream withMapper: mapper
	self writeMethodHeaderOn: aByteStream.
	self primitive ifNil: [
		self literals do: [ :literal | 
			literal isInteger 
				ifTrue: [ aByteStream littleEndianWord: (literal asNXTSmallInteger) ]
				ifFalse: [ aByteStream littleEndianWord: (mapper idFor: literal) ]].
		self bytecodes do: [ :bytecode | aByteStream nextPut: bytecode ]].! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:03'!
writeMethodHeaderOn: aByteStream
	aByteStream littleEndianWord: self methodHeader.! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 16:42'!
computeSize
	^ self instanceVariables size! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 16:44'!
elementSize
	^ 2! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:19'!
header
	| header |
	"TODO this code assumes, size is below 2048"
	header := ((self elementSize bitAnd: 3) << 12 bitOr: 1 << 11) bitOr: (self computeSize bitAnd: 16r7FF).
	^ header! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 17:34'!
initialize
	nxtClass := #NXTSystem.
	instanceVariables := OrderedCollection new.! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:03'!
instanceVariables
	^ instanceVariables! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:03'!
instanceVariables: anObject
	instanceVariables := anObject! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 4/28/2008 16:03'!
nxtClass
	^ nxtClass ifNil: [ #NXTSystem ]! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:04'!
nxtClass: anObject
	nxtClass := anObject! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 16:33'!
writeDataOn: aByteStream withMapper: mapper
	self instanceVariables do: [ :instanceVariable |
		aByteStream littleEndianWord: (mapper idFor: instanceVariable) ].! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:39'!
writeHeaderOn: aByteStream
	aByteStream littleEndianWord: self header.! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 14:51'!
writeOn: aByteStream withMapper: mapper
	self writeHeaderOn: aByteStream.
	aByteStream littleEndianWord: (mapper idFor: self nxtClass).
	self writeDataOn: aByteStream withMapper: mapper.! !

NXTImageObject subclass: #NXTImageSymbol
	instanceVariableNames: 'symbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageSymbol methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:06'!
computeSize
	^ self symbol size! !

!NXTImageSymbol methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:06'!
elementSize
	^ 1! !

!NXTImageSymbol methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:09'!
symbol
	^ symbol! !

!NXTImageSymbol methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:09'!
symbol: anObject
	symbol := anObject! !

!NXTImageSymbol methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:08'!
writeDataOn: aByteStream withMapper: mapper
	aByteStream nextPutAll: self symbol! !

Object subclass: #NXTObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTArray subclass: #NXTBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 16:51'!
restart
	<NXTPrimitive: 5>
	self restart! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 18:15'!
value
	<NXTPrimitive: 2>! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 16:43'!
whileTrue: aBlock
	self value ifFalse: [ ^ nil ].
	aBlock value.
	self restart.! !

NXTArray subclass: #NXTCompiledMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTBoolean
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTBoolean subclass: #NXTFalse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:28'!
& aBoolean
	^ self! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:16'!
ifFalse: aBlock
	^ aBlock value! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/15/2008 13:46'!
ifFalse: falseBlock ifTrue: trueBlock
	^ falseBlock value! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:16'!
ifTrue: aBlock
	^ nil! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 21:41'!
ifTrue: trueBlock ifFalse: falseBlock
	^ self ifFalse: falseBlock ifTrue: trueBlock! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:28'!
not
	^ true! !

NXTBoolean subclass: #NXTTrue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:29'!
& aBoolean
	^ aBoolean! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:17'!
ifFalse: aBlock
	^ nil! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/15/2008 13:46'!
ifFalse: falseBlock ifTrue: trueBlock
	^ trueBlock value! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:17'!
ifTrue: aBlock
	^ aBlock value! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 21:41'!
ifTrue: trueBlock ifFalse: falseBlock
	^ self ifFalse: falseBlock ifTrue: trueBlock! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:28'!
not
	^ false! !

NXTObject subclass: #NXTClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Collections'!

NXTObject subclass: #NXTMetaclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTSmallInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 15:25'!
+ other
	<NXTPrimitive: 3>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 21:46'!
< other
	<NXTPrimitive: 11>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 17:40'!
= other
	<NXTPrimitive: 7>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:38'!
bitAnd: other
	<NXTPrimitive: 8>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:55'!
isZero
	^ self = 0! !

NXTObject subclass: #NXTString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTString methodsFor: 'as yet unclassified' stamp: 'mab 3/11/2008 01:09'!
printNl
	<primitive: 500>! !

NXTString subclass: #NXTSymbol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTSystem
	instanceVariableNames: 'running enterPressed driving right left'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTSystem methodsFor: 'button reading' stamp: 'mab 4/29/2008 17:36'!
buttonRead
	<NXTPrimitive: 6>! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 5/7/2008 17:55'!
checkEnterDown: key
	(key bitAnd: 8) isZero not & enterPressed not ifTrue: [
		enterPressed := true ].! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 5/15/2008 15:04'!
checkEnterUp: key
	(key bitAnd: 8) isZero & enterPressed ifTrue: [
		enterPressed := false.
		driving 
			ifFalse: [ self printString: 'Start driving'.
				self driveAhead.
				driving := true ]
			ifTrue: [  self printString: 'Stop driving'.
				self stopDriving.
				driving := false ]].! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 5/16/2008 19:14'!
checkLeft: key
	(key bitAnd: 4) isZero not & left not ifTrue: [
		left := true.
		self driveLeft ].
	(key bitAnd: 4) isZero & left ifTrue: [
		left := false.
		self driveAhead ].! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 5/15/2008 16:53'!
checkRight: key
	(key bitAnd: 2) isZero not & right not & left not ifTrue: [
		right := true.
		self driveRight ].
	(key bitAnd: 2) isZero & right ifTrue: [
		right := false.
		self driveAhead ].! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 17:01'!
driveAhead
	self motor: 0 mode: 1.
	self motor: 1 mode: 1.
	self motor: 0 speed: 50.
	self motor: 1 speed: 50.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/16/2008 21:42'!
driveBack
	self motor: 0 mode: 1.
	self motor: 1 mode: 1.
	self motor: 0 speed: -50.
	self motor: 1 speed: -50.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 16:27'!
driveLeft
	self motor: 0 mode: 0.
	self motor: 0 speed: 0.
	self motor: 1 mode: 1.
	self motor: 1 speed: 50.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 16:28'!
driveRight
	self motor: 0 mode: 1.
	self motor: 0 speed: 50.
	self motor: 1 mode: 0.
	self motor: 1 speed: 0.! !

!NXTSystem methodsFor: 'initialize-release' stamp: 'mab 5/17/2008 10:07'!
initialize
	self stopDriving.
	running := true.
	enterPressed := false.
	driving := false.
	left := false.
	right := false.
	! !

!NXTSystem methodsFor: 'system' stamp: 'mab 5/16/2008 20:37'!
keyLoop
	| key |
	key := self buttonRead.
	key = 1 ifTrue: [ running := false ].
	self checkEnterDown: key.
	self checkEnterUp: key.
	self checkLeft: key.
	self checkRight: key.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 14:51'!
motor: aNumber mode: aMode
	<NXTPrimitive: 9>! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 14:51'!
motor: aNumber speed: aSpeed
	<NXTPrimitive: 1>! !

!NXTSystem methodsFor: 'printing' stamp: 'mab 4/29/2008 15:25'!
printNumber: aSmallInt
	<NXTPrimitive: 4>
	Transcript show: aSmallInt asString; cr.! !

!NXTSystem methodsFor: 'printing' stamp: 'mab 4/21/2008 15:34'!
printString: aString
	<NXTPrimitive: 0>
	Transcript show: aString; cr.! !

!NXTSystem methodsFor: 'system' stamp: 'mab 5/17/2008 10:08'!
run
	self initialize.
	self printString: 'Entering driving mode now:'.
  	self printString: 'Enter to start and stop engine'.
  	self printString: 'Left/Right for steering'.
	[ running ] whileTrue: [ self keyLoop ].
	self printString: 'Goodbye!!'.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 14:58'!
stopDriving
	self motor: 0 mode: 0.
	self motor: 1 mode: 0.
	self motor: 0 speed: 0.
	self motor: 1 speed: 0.! !

!NXTSystem methodsFor: 'time' stamp: 'mab 5/16/2008 21:32'!
time
	<NXTPrimitive: 10>! !

!NXTSystem methodsFor: 'system' stamp: 'mab 5/20/2008 11:50'!
timeLoop
	self buttonRead = 1 ifTrue: [ running := false ].
	self time < 15
		ifTrue: [ self driveAhead ]
		ifFalse: [ self time < 20
			ifTrue: [ self stopDriving ]
			ifFalse: [ self time < 30
				ifTrue: [ self driveBack ]
				ifFalse: [ self stopDriving. running := false ]]]! !

NXTObject subclass: #NXTUndefinedObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

Object subclass: #NXTObjectMapper
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTObjectMapper class methodsFor: 'as yet unclassified' stamp: 'mab 3/17/2008 17:15'!
on: objectList
	^ self new objects: objectList! !

!NXTObjectMapper methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 00:30'!
idFor: anObject
	anObject 
		ifNotNil: [ ^ (self objects indexOf: anObject ifAbsent: [ self error: 'Non-mappable object' ]) - 1 ]! !

!NXTObjectMapper methodsFor: 'accessing' stamp: 'mab 3/17/2008 17:15'!
objects
	^ objects! !

!NXTObjectMapper methodsFor: 'accessing' stamp: 'mab 3/17/2008 17:15'!
objects: anObject
	objects := anObject! !

!PositionableStream methodsFor: '*nxtalk' stamp: 'mab 4/12/2008 14:24'!
littleEndianWord
	self nextLittleEndianNumber: 2! !

!PositionableStream methodsFor: '*nxtalk' stamp: 'mab 4/12/2008 14:24'!
littleEndianWord: number
	self nextLittleEndianNumber: 2 put: number! !

!String methodsFor: '*nxtalk' stamp: 'mab 3/11/2008 01:09'!
printNl
	Transcript show: self; cr.! !

TestCase subclass: #NXTObjectMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Tests'!
